BagSlots = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
BagSlots:RegisterDB("BagSlotsDB")

function BagSlots:OnInitialize()
	self.crayon = AceLibrary("Crayon-2.0")

	BagSlots:RegisterChatCommand({ "/bagslots", "/bslots" }, {
		desc = "Display bag usage on each of your bag slots.",
		type = 'group',
		args = {},
	})

	self.bags = self.bags or {
		[1] = "MainMenuBarBackpackButton",
		[2] = "CharacterBag0Slot",
		[3] = "CharacterBag1Slot",
		[4] = "CharacterBag2Slot",
		[5] = "CharacterBag3Slot"
	}
    
	BagSlots:PrepOverlay()
end

-- Start
function BagSlots:OnEnable()
	self:RegisterEvent("BAG_UPDATE", "UpdateSlotCount")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnWorldEnter")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "OnWorldLeave")
	self:UpdateSlotCount()
end

function BagSlots:OnDisable()
	for _,bag in pairs(self.bags) do
		local BPSlot = getglobal(bag.."UsageString")
		BPSlot:SetText("")
	end
end

function BagSlots:OnWorldEnter()
	self:RegisterEvent("BAG_UPDATE", "UpdateSlotCount")
end

function BagSlots:OnWorldLeave()
	self:UnregisterEvent("BAG_UPDATE")
end

function BagSlots:PrepOverlay()
	for _,bag in ipairs(self.bags) do
		local BPSlot = getglobal(bag)
		local BPStr = BPSlot:CreateFontString(bag.."UsageString", "ARTWORK")
		BPStr:ClearAllPoints()
		BPStr:SetFontObject(GameFontHighlightSmallOutline)
		BPStr:SetPoint("CENTER", BPSlot, "BOTTOM", 0, 6)
	end	
end

-- Storage for stuff
local numSlots = {0,0,0,0,0}
local usedSlots = {0,0,0,0,0}

function BagSlots:UpdateSlotCount()
	local totalSlots = 0
	local totalUsedSlots = 0
	local slotsText = 0
	
	for bag = 0, 4 do
		numSlots[bag+1] = GetContainerNumSlots(bag)

		if (numSlots[bag+1] == 0) then
			return
		else
			totalSlots = totalSlots + numSlots[bag+1]
			usedSlots[bag+1] = 0
			for slot = 1, numSlots[bag+1] do
				if (GetContainerItemInfo(bag, slot)) then
					usedSlots[bag+1] = usedSlots[bag+1] + 1
					totalUsedSlots = totalUsedSlots + 1
				end
			end
			
			slotsText = usedSlots[bag+1].."/"..numSlots[bag+1]
		end
		-- Set the usage string
		local bagslot = getglobal(self.bags[bag+1].."UsageString")
		SetItemButtonCount(getglobal(self.bags[bag+1]), nil)
		bagslot:SetTextColor(self.crayon:GetThresholdColor(usedSlots[bag+1]/numSlots[bag+1], 1, 0.8, 0.6, 0.4, 0.2))
		bagslot:SetText(slotsText)
	end
end
